/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.sample;

import net.disy.ogc.wps.v_1_0_0.annotation.InputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.Metadata;
import net.disy.ogc.wps.v_1_0_0.annotation.OutputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.Process;
import net.disy.ogc.wps.v_1_0_0.annotation.ProcessMethod;
import net.opengis.ows.v_1_1_0.BoundingBoxType;

import org.apache.commons.lang.Validate;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LinearRing;
import com.vividsolutions.jts.geom.Polygon;

@Process(
//
id = "boundingBoxPolygon",
//
title = "Bounding box to polygon",
//
description = "Returns current bounding box as polygon",
//
metadata = {
    @Metadata(about = "tag", href = "wps-demo:form"),
    @Metadata(about = "tag", href = "wps-demo:toolbar") })
public class BoundingBoxPolygonAnnotatedObject {
  @ProcessMethod
  @OutputParameter(
  //
  id = "polygon",
  //
  title = "Polygon",
  //
  description = "Polygon for the current bounding box")
  public Polygon boundingBoxToPolygon(@InputParameter(
  //
  id = "boundingBox",
  //
  title = "Bounding box",
  //
  description = "Current") BoundingBoxType input) {

    Validate.notNull(input);
    Validate.isTrue(input.isSetLowerCorner());
    Validate.isTrue(input.getLowerCorner().size() >= 2);
    Validate.isTrue(input.getLowerCorner().get(0) != null);
    Validate.isTrue(input.getLowerCorner().get(1) != null);
    Validate.isTrue(input.getUpperCorner().size() >= 2);
    Validate.isTrue(input.getUpperCorner().get(0) != null);
    Validate.isTrue(input.getUpperCorner().get(1) != null);

    final double _x1 = input.getLowerCorner().get(0).doubleValue();
    final double _y1 = input.getLowerCorner().get(1).doubleValue();
    final double _x2 = input.getUpperCorner().get(0).doubleValue();
    final double _y2 = input.getUpperCorner().get(1).doubleValue();

    final double x1 = Math.min(_x1, _x2);
    final double y1 = Math.min(_y1, _y2);
    final double x2 = Math.max(_x1, _x2);
    final double y2 = Math.max(_y1, _y2);

    final Coordinate[] coordinates = new Coordinate[]{
        new Coordinate(x1, y1),
        new Coordinate(x2, y1),
        new Coordinate(x2, y2),
        new Coordinate(x1, y2),
        new Coordinate(x1, y1) };

    final GeometryFactory geometryFactory = new GeometryFactory();
    return geometryFactory.createPolygon(
        geometryFactory.createLinearRing(coordinates),
        new LinearRing[0]);
  }

}
